Unit Animate;
{
  Simple, almost mindless, handlers for animation.
}

Interface

Procedure SaveScreenView;
Procedure PlayScreenViewFile;
Procedure DoAnimation;

Implementation

Uses
  CRT,         { keyressed function for faster bails }
  DOS,         { for FileRec structure }
  Graph,       { graphics routines }
  Graphics,
  GrText,
  Print,
  Vectors,     { vector operations and Float declaration }
  FDecls,      { most global declarations }
  ParsEval;    {  }

Const
  FrameSize = 64000;      { size of a frame in the file }

{-------------------------------------------------------------------------}

Procedure DoBlur;
{
  Perform 3x3 pixel averaging on the screen
}
Type
  ByteArray = Array[0..1] of Byte;
Var
  X, Y,
  dX, dY,
  I, J,
  A : Integer;
  SX, SY : Word;
  P : ^ByteArray;
Begin

  If MaxAvail >= FrameSize then

    { allocate memory for scratch buffer }
    GetMem(P, FrameSize);

    { clear the dest buffer }
    FillChar(P^, FrameSize, 0);

    { blur the image }
    For Y := 0 to 199 do begin
      For X := 0 to 319 do begin

        A := 0;

        For dY := -1 to 1 do begin
          SY := Y + dY;
          If SY <= 199 then begin
            SY := SY * 320;
            For dX := -1 to 1 do begin
              SX := X + dX;
              If SX <= 319 then begin
                A := A + Mem[$A000:SY + SX] - ColorBase;
              End { If }
            End { For }
          End { If }
        End; { For }

        A := A div 9;
        If A < 0 then
          A := 0
        Else If A > 63 then
          A := 63;

        P^[Y * 320 + X] := A + ColorBase;

      End { For }
    End; { For }

    { copy the image back to the screen }
    Move(P^, Mem[$A000:$0000], FrameSize);

    { free up the scratch buffer }
    FreeMem(P, FrameSize);

End; { Procedure DoBlur }

{---------------------------------------------------------------------------}

Procedure SaveScreenView;
{
  Save the current image on the 320x200x256 color screen to the file
  specified by ScreenViewName.
}

Var
  OutFile : File;
  L : LongInt;

Begin

  {$I-}

  { if in the correct video mode }
  If (Mem[$0:$449] = $13) then begin

    { attempt to open the file specified }
    Assign(OutFile, AnimName);
    Reset(OutFile);

    { if the file was found }
    If IOResult = 0 then begin

      FileRec(OutFile).RecSize := 1;

      L := FileSize(OutFile);

      { find the end of the file }
      Seek(OutFile, FileSize(OutFile));

      { if a frame was specified and in the file, seek to its start }
      If AnimFrame >= 0 then
        If FileSize(OutFile) >= LongInt(AnimFrame) * LongInt(FrameSize) then
          Seek(OutFile, LongInt(AnimFrame) * LongInt(FrameSize));

    End { If }

    { wasn't found, open a new one }
    Else
      Rewrite(OutFile);

    { blur the image as required }
    If Blur then
      DoBlur;

    { write the screen to the file }
    FileRec(OutFile).RecSize := 1;
    BlockWrite(OutFile, Mem[$A000:$0000], FrameSize);

    { close the file }
    Close(OutFile);

  End { If }

End; { Procedure SaveScreenView }

{-------------------------------------------------------------------------}

Procedure PlayScreenViewFile;
{
  Plays the file specified by ScreenViewName.  Assumes this is a simple
  bitmap form containing a set of 320x200x256 color screens.
}
Var
  InFile    : File;     { file to read from }
  n         : Word;     { # bytes read from file }
  L,
  Offset,               { current offset into the file }
  dOffset,              { amount to move file offset to get to the next frame }
  MaxOffset : LongInt;  { maximum value of Offset allowed }
  TBuf      : Pointer;  { pointer to temporary buffer to hold the frame }
Begin

  { if in the correct video mode }
  If (Mem[$0:$449] = $13) then begin

    { attempt to open the file specified }
    Assign(InFile, AnimName);
    {$I-} Reset(InFile);

    { if the file was found }
    If IOResult = 0 then begin

      { Set the file record size to 1 byte to make the math easier to handle }
      FileRec(InFile).RecSize := 1;

      { determine frame-aligned file size }
      MaxOffset := (FileSize(InFile) div FrameSize) * LongInt(FrameSize);

      { start at the beginning of the file }
      Offset := 0;

      { how far in the file to go to read the next frame }
      dOffset := FrameSize;

      { allocate a temporary buffer if possible, or just set it to the screen }
      If MaxAvail < FrameSize then
        TBuf := Pointer($A0000000)
      Else
        GetMem(TBuf, FrameSize);

      { while operator hasn't pressed a key }
      While not KeyPressed do begin

        L := FilePos(InFile);

        { read the current frame data from file to temp location }
        BlockRead(InFile, TBuf^, FrameSize, n);

        { copy the temp frame buffer to screen mem }
        If LongInt(TBuf) <> $A0000000 then
          Move(TBuf^, Mem[$A000:$0000], FrameSize);

        { adjust the index into the file }
        Offset := Offset + dOffset;

        { if past the end of the file or past the beginning of the file }
        If (Offset >= MaxOffset) or (Offset < 0) then begin

          { if reversing direction, reverse the offset and adjust the index }
          If AnimReverse then begin
            dOffset := -dOffset;
            Offset  := Offset + dOffset;
          End { If }

          { else reset the index to 0 }
          Else begin
            dOffset := FrameSize;
            Offset  := 0;
          End; { Else }

        End; { If }

        { seek the correct frame in the file }
        Seek(InFile, Offset);

        { wait the amount the user said to }
        If AnimDelay > 0 then
          Delay(AnimDelay);

      End; { While }

      { close the file now tht we're done with it }
      Close(InFile);

      { dump the temp frame buffer }
      If LongInt(TBuf) <> $A0000000 then
        FreeMem(TBuf, FrameSize);

    End { If }
    Else begin
      If Pos('.DAT', AnimName) <> 0 then
        Abort('Unable to locate file ' + AnimName + '!')
      Else begin
        AnimName := AnimName + '.DAT';
        PlayScreenViewFile;
      End; { Else }
    End; { Else }
  End { If }
  Else
    Abort('Wrong Video mode for playback (must be 320x200 256 color)!');

  Abort(NAbortPrefix);

End; { Procedure PlayScreenViewFile }

{-------------------------------------------------------------------------}

Procedure DoAnimation;
{
  Performs the animation defined by the current animation setting parameters.
}

Const
  VPX = 'XXXANIMVPOINTX';
  VPY = 'XXXANIMVPOINTY';
  VPZ = 'XXXANIMVPOINTZ';
  FPX = 'XXXANIMFPOINTX';
  FPY = 'XXXANIMFPOINTY';
  FPZ = 'XXXANIMFPOINTZ';

Var
  dParm : Float;   { difference in Parm between frames }
  Frame : Integer; { current frame number }
  Error,           { parsing error number }
  Posn : Integer;  { parsing error position }
  OutFile : File;  { output file }

Begin

  { make sure there is at least one frame in the calculation }
  If AnimFrames = 0 then
    AnimFrames := 1;

  { set up the animation parameters }
  dParm := (AnimStop - AnimStart) / AnimFrames;
  AnimParm := AnimStart;

  { line up the correct viewpoint info if required }
  If ViewParse then begin

    Parse(VPX + ':=' + ViewText[1], Error, Posn);
    If (Error <> NoError) and (Error < LastError) then
      Abort('Error parsing animation viewpoint X coordinate');
    Parse(VPY + ':=' + ViewText[2], Error, Posn);
    If (Error <> NoError) and (Error < LastError) then
      Abort('Error parsing animation viewpoint Y coordinate');
    Parse(VPZ + ':=' + ViewText[3], Error, Posn);
    If (Error <> NoError) and (Error < LastError) then
      Abort('Error parsing animation viewpoint Z coordinate');

  End; { If }

  { line up the focus point info if required }
  If FocusParse then begin

    Parse(FPX + ':=' + FocusText[1], Error, Posn);
    If (Error <> NoError) and (Error < LastError) then
      Abort('Error parsing animation focus X coordinate');
    Parse(FPY + ':=' + FocusText[2], Error, Posn);
    If (Error <> NoError) and (Error < LastError) then
      Abort('Error parsing animation focus Y coordinate');
    Parse(FPZ + ':=' + FocusText[3], Error, Posn);
    If (Error <> NoError) and (Error < LastError) then
      Abort('Error parsing animation focus Z coordinate');

  End; { If }

  { open the file }

  If AnimName = '' then
    Abort('Please use the ANIMSAVE command to name an output file');

  Assign(OutFile, AnimName);
  Rewrite(OutFile);
  If IOResult <> 0 then
    Abort('Unable to open file ''' + AnimName + '''.');
  FileRec(OutFile).RecSize := 1;

  { for each frame in the animation }
  Frame := 1;
  While (Frame <= AnimFrames) and (Ch <> #27) do begin

    { set the screen frame number for the save }
    AnimFrame := Frame - 1;

    { if we need to parse the viewpoint }
    If ViewParse then begin

      ViewPoint.X := GetVarValue(VPX);
      If ViewPoint.X = ConstError then
        Abort('Error evaluating animation viewpoint X coordinate');
      ViewPoint.Y := GetVarValue(VPY);
      If ViewPoint.Y = ConstError then
        Abort('Error evaluating animation viewpoint Y coordinate');
      ViewPoint.Z := GetVarValue(VPZ);
      If ViewPoint.Z = ConstError then
        Abort('Error evaluating animation viewpoint Z coordinate');

    End; { If }

    { if we need to parse the focus point }
    If FocusParse then begin

      { recompute the focus point }
      Focus.X := GetVarValue(VPX);
      If Focus.X = ConstError then
        Abort('Error evaluating animation focus X coordinate');
      Focus.Y := GetVarValue(VPY);
      If Focus.Y = ConstError then
        Abort('Error evaluating animation focus Y coordinate');
      Focus.Z := GetVarValue(VPZ);
      If Focus.Z = ConstError then
        Abort('Error evaluating animation focus Z coordinate');

    End; { If }

    { clear the screen }
    FillChar(Mem[$A000:$0000], FrameSize, 0);

    { draw the surface }
    DrawGrid(XGridSize, YGridSize, Xmin, YMin, XMax, YMax);

    { draw user text }
    DrawText(UserTextXPos, UserTextYPos, UserTextZPos,
             UserTextXRot, UserTextYRot, UserTextZRot, UserText);

    { Eject the printer page if required }
    EjectPage;

    { don't save the image if no data }
    If Ch <> #27 then begin

      { blur the image as required }
      If Blur then
        DoBlur;

      { write out the screen memory to the file }
      BlockWrite(OutFile, Mem[$A000:$0000], FrameSize);

    End; { If }

    { update the animation parameter for the next frame }
    AnimParm := AnimParm + dParm;

    { update the frame number }
    Frame := Frame + 1;

  End; { While }

  { close the output file on the way past }
  Close(OutFile);

End; { Procedure DoAnimation }

{-------------------------------------------------------------------------}

End.