Unit RayCast;

(****************************************************************************
**                                                                         **
**  Ray-casting regndering routines.                                       **
**                                                                         **
****************************************************************************)

Interface

Uses
  Vectors;     { coordinate definitions }


Procedure RayDrawGrid(XGridSize, YGridSize : Word;
                      GridXMin, GridYMin,
                      GridXMax, GridYMax : Float);

Implementation

Uses
  CRT,       { KeyPressed }
  Print,     { for printer type determinations }
  GrText,    { stroke text stuff }
  View,      { viewing transformation thingy }
  GDrivers,  { graphics drivers }
  FDecls,    { abort routine and general declarations }
  Triangle,  { triangle drawing routine and associated fillers }
  ParsEval;  { evaluation routine }

{---------------------------------------------------------------------------}

Procedure RayDrawGrid;
{
  draw grid on screen using a ray-casting algorithm

}
Begin

{
  Required Inputs:
    Field-of-view angle        scalar, degrees
    Direction of view          vector
    Starting column of screen  scalar, pixels
    Starting row of screen     scalar, pixels
    Width of screen            scalar, pixels
    Height of screen           scalar, pixels
    X bounding box min         scalar
    Y bounding box min         scalar
    Z bounding box min         scalar
    X bounding box max         scalar
    Y bounding box max         scalar
    Z bounding box max         scalar
    Precomputed Height field
    [an alternate to the height field is to evalute the function at each u,v
     step during the evaluation.  Would yield a better display, but would be
     much more computation intensive.]


  Outputs:
    Screen image drawn onscreen

}

  

  for each column on the screen

    determine the angle away from the viewpoint ((FOV / Width) / 2)

    determine the u,v step size based on grid step size and view dir

    find the first intersection with the box

    start the screen Y max at 0 and the Y min at

    while still within the height field

      determine Z at u,v (preferably interpolated)

      rotate Z to the screen coordinate system

      if screen Y is higher than the previous max high,

        determine the color at this point

        draw a line to the old posn in the calculated color

        set the new max high to the screen Y

      else if the screen Y is lower than the previous min low

        determine the color at this point

        draw a line to the old posn in the calculated color

        set the new min low to the screen Y

      step the u,v to the next position


End; { Procedure RayDrawGrid }


End.